use strict;
use warnings;

package Perl::Critic::Lax;

=head1 NAME

Perl::Critic::Lax - policies that let you slide on common exceptions

=head1 VERSION

version 0.006

  $Id: /my/cs/projects/Perl-Critic-Lax/trunk/lib/Perl/Critic/Lax.pm 32194 2007-07-27T23:18:32.278761Z rjbs  $

=cut

our $VERSION = '0.006';

=head1 DESCRIPTION

The Perl-Critic-Lax distribution includes versions of core Perl::Critic modules
with built-in exceptions.  If you really like a Perl::Critic policy, but find
that you often violate it in a specific way that seems pretty darn reasonable,
maybe there's a Lax policy.  If there isn't, send one in!

=head1 AUTHOR

Ricardo SIGNES <rjbs@cpan.org>

=head1 COPYRIGHT

Copyright (c) 2006 Ricardo SIGNES, Chris Dolan, Jeffrey Ryan Thalhammer.

This program is free software; you can redistribute it and/or modify it under
the same terms as Perl itself.

=cut

1;
